! ********************************************************************
!               Copyright 1990-1995 by David Lee Cohen
!                    All Rights Reserved
!          Licensed Material - Property of David Lee Cohen
!
!                    Version Date: 5/8/95
! ********************************************************************
!
! ********************************************************************
! Calculator and Calendar 
! ********************************************************************
!
! The Calculator() code implements a calculator featuring:
!   * Unlike many little Windows calculators, you can use the number
!     keys or numeric keypad in addition to the mouse.
!   * Add, subtract, multiply divide operations.
!   * Negate function.
!   * Power and Square Root function.
!   * Copy and Paste between Windows Clipboard.
!   * Single persistant memory.
!
! The Calendar() code implements a calendar featuring:
!   * Full month display.
!   * Flags current date.
!   * Move forward or ahead one month or one year.
!   * Current time.
!
! ********************************************************************
! Cost, Restrictions, and Some Legal Stuff
! ********************************************************************
!
! The Calculator and Calendar code are offered to Clarion for Windows
! developers as shareware. All I ask is a donation of $10 for both.
! Please send payment to:
!   AlleyCat Software, Inc.
!   P.O. Box 14446
!   RTP, NC  27709-4446
!   Compuserve: 70431,132
!   Mastercard/Visa Accepted
!
! What you get is the right to use the code within your applications 
! as you please, with no limits on distribution. You are free to alter
! it in any way you see fit to work with your application.
!
! However, you can not resell or distribute the Calculator and 
! Calendar code as is or with cosmetic changes only, without first 
! receiving permission from David Lee Cohen and AlleyCat Software, Inc.
!
! This code is offered as is, and neither David Lee Cohen or Alleycat
! Software, Inc. is responsible for defects or damages.
! 
! ********************************************************************
! How to Use
! ********************************************************************
!
! The Calculator and Calendar code is simple to use. Each is a self-
! contained chunk of code that doesn't depend on any external 
! variables or procedures. There are no parameters, so they can be
! STARTed in their own threads. 
!
! I'm primarily a hand-coder. Thus, you can use these procedures in
! the Application Generator as Source Procedures, or make some minor
! tweaks and turn them into Code or Procedure Templates.
! 
! There are a few changes you'll need to make to use the Calculator
! and Calendar within your applications:
!
! 1. Change the MEMBER() statement to reference your program.
!    Notice that I've also included a PROGRAM header that can
!    turn these procedures into standalone programs.
!
! 2. Set the IS_PROGRAM conditional compile EQUATE:
!    IS_PROGRAM = 1 means a complete program will be built.
!    IS_PROGRAM = 0 means a MEMBER module is used. You'll need to
!      add the prototype to your program's MAP:
!      MAP
!        MODULE('CALC.CLW')
!          Calculator
!        END
!        MODULE('CALEND.CLW')
!          Calendar
!        END
!      END
!
! 3. Set the USE_LOCAL_FUNCS conditional compile EQUATE:
!    USE_LOCAL_FUNCS = 1 means that a couple local functions are used.
!      I have these defined in libraries, so don't include them here.
!
! 4. Set the USE_THREADS conditional compile EQUATE:
!    USE_THREADS = 1 means that I call my own simple thread management
!      routines. You may want to insert your own thread management 
!      calls at these points instead.
!
! In my applications, I have:
!   IS_PROGRAM = 0
!   USE_LOCAL_FUNCS = 0
!   USE_THREADS = 1
! In your applications, you will likely have:
!   IS_PROGRAM = 0
!   USE_LOCAL_FUNCS = 1
!   USE_THREADS = 1
!   Plus, you will have some minor tweaks to thread code.
! For a standalone program, you would have:
!   IS_PROGRAM = 1
!   USE_LOCAL_FUNCS = 1
!   USE_THREADS = 0
!   (Plus, uncomment out program header below.)
!
! All in all, the Calculator and Calendar procedures should be very
! easy to just plug into your applications. I've been using them
! for some time now - they actually evolved from CDD 3.0 versions.
! Please report any bugs or problems to me, as well as any nifty
! new ideas you dream up.
!
! ********************************************************************
! Files
! ********************************************************************
!
! You should have these files:
!   CALC.CLW - Calculator source code
!   CALC.ICO - Calculator icon
!   CALEND.CLW - Calendar source code
!   CALEND.ICO - Calendar icon
!
! ********************************************************************

! Clarion doesn't allow conditional compilation for PROGRAM/MEMBER.
! So, comment from BEGIN_PROGRAM_HEADER/END_PROGRAM_HEADER for member,
! or comment BEGIN_MEMBER_HEADER/END_MEMBER_HEADER for program.

!BEGIN_PROGRAM_HEADER
!                 TITLE('CALCULATOR')
!                 PROGRAM
!
!                 INCLUDE('KEYCODES')
!                 INCLUDE('EQUATES')
!END_PROGRAM_HEADER

!BEGIN_MEMBER_HEADER
                 MEMBER('CAPGNZ')
!END_MEMBER_HEADER

! Is this a standalone program?
! If IS_PROGRAM=1, then Program Header above should be uncommented.
! If IS_PROGRAM=0, then Member Header above should be uncommented.
IS_PROGRAM              EQUATE(0) 

! Use local lenclip(), roundoff() ?
! I have these in my own libraries, so don't need them.
USE_LOCAL_FUNCS         EQUATE(0) 

! I use a simple thread management scheme here.
! Calls to SetThread(), RestoreThread(), and ClearThreadNumber()
! are used. You can replace these with your own thread mgmt calls, if
! necessary.
USE_THREADS             EQUATE(1) 

  MAP
    SelectPress(SHORT)
  COMPILE('LOCAL_FUNCS_DEF', USE_LOCAL_FUNCS=1)
    lenclip(STRING),SHORT
    roundoff(REAL,REAL),REAL
LOCAL_FUNCS_DEF
  COMPILE('CALCULATOR_DEF', IS_PROGRAM=1)
    Calculator
CALCULATOR_DEF
  END

CalcMem         REAL(0) ! Static memory

  COMPILE('PROGRAM_CODE', IS_PROGRAM=1)
  CODE
  Calculator()
PROGRAM_CODE

Calculator              PROCEDURE

! Key equates. Standard Clarion EQUATES are available for most, but
! curiously not all, of these.
KeyNum0         EQUATE(48)
KeyNum1         EQUATE(49)
KeyNum2         EQUATE(50)
KeyNum3         EQUATE(51)
KeyNum4         EQUATE(52)
KeyNum5         EQUATE(53)
KeyNum6         EQUATE(54)
KeyNum7         EQUATE(55)
KeyNum8         EQUATE(56)
KeyNum9         EQUATE(57)
KeyPadNum0      EQUATE(96)
KeyPadNum1      EQUATE(97)
KeyPadNum2      EQUATE(98)
KeyPadNum3      EQUATE(99)
KeyPadNum4      EQUATE(100)
KeyPadNum5      EQUATE(101)
KeyPadNum6      EQUATE(102)
KeyPadNum7      EQUATE(103)
KeyPadNum8      EQUATE(104)
KeyPadNum9      EQUATE(105)
KeySpace        EQUATE(32)
KeyEqual        EQUATE(187)
KeyPlus         EQUATE(443)
KeyPadPlus      EQUATE(107)
KeyMinus        EQUATE(189)
KeyPadMinus     EQUATE(109)
KeyMult         EQUATE(312)
KeyPadMult      EQUATE(106)
KeyDiv          EQUATE(191)
KeyPadDiv       EQUATE(111)
KeyDecPt        EQUATE(190)
KeyPadDecPt     EQUATE(110)

EraseField      EQUATE(255)

! If change these, must change ValueStr routine, too
MAX_DEC      EQUATE(10) ! Max decimal digits
MAX_DIGITS   EQUATE(14) ! Max total digits, not including dec pt or neg sign
MAX_VALSTR   EQUATE(17) ! Max value string length (dec,neg,0)
MAX_DEC_RND  EQUATE(.0000000001)
FMT_MAX_DIGITS STRING(@P<<<<<<<<<<<<<#P)

ValStr       STRING(MAX_VALSTR) ! Actual number displayed on calculator
MemInd       STRING(1)          ! Memory indicator

  COMPILE('MDI_DEF', USE_THREADS=1)
Screen WINDOW('Calculator'),AT(,,111,132),ICON('CALC.ICO'),GRAY,MDI
MDI_DEF
  COMPILE('NONMDI_DEF', USE_THREADS=0)
Screen WINDOW('Calculator'),AT(,,111,132),ICON('CALC.ICO'),GRAY
NONMDI_DEF
       BOX,AT(4,2,104,14),COLOR(00H)
       STRING(@s1),AT(8,4,,),USE(MemInd)
       STRING(@s20),AT(24,4,80,10),USE(ValStr),RIGHT
       BUTTON('&Clr'),AT(4,20,24,14),USE(?Clear)
       BUTTON('Cop&y'),AT(32,20,24,14),USE(?Copy)
       BUTTON('Po&w'),AT(64,20,16,14),USE(?Power)
       GROUP('Mem'),AT(84,28,24,76),BOXED
         BUTTON('&Add'),AT(88,40,16,14),USE(?AddMem)
         BUTTON('&Sub'),AT(88,56,16,14),USE(?SubMem)
         BUTTON('&Flsh'),AT(88,72,16,14),USE(?FlushMem)
         BUTTON('&Rcll'),AT(88,88,16,14),USE(?RecallMem)
       END
       BUTTON('Clr &En'),AT(4,36,24,14),USE(?ClearEntry)
       BUTTON('&Paste'),AT(32,36,24,14),USE(?Paste)
       BUTTON('S&qrt'),AT(64,36,16,14),USE(?Sqrt)
       BUTTON('7'),AT(4,52,16,14),USE(?Key7)
       BUTTON('8'),AT(24,52,16,14),USE(?Key8)
       BUTTON('9'),AT(44,52,16,14),USE(?Key9)
       BUTTON('+'),AT(64,52,16,14),USE(?Plus)
       BUTTON('4'),AT(4,68,16,14),USE(?Key4)
       BUTTON('5'),AT(24,68,16,14),USE(?Key5)
       BUTTON('6'),AT(44,68,16,14),USE(?Key6)
       BUTTON('-'),AT(64,68,16,14),USE(?Minus)
       BUTTON('1'),AT(4,84,16,14),USE(?Key1)
       BUTTON('2'),AT(24,84,16,14),USE(?Key2)
       BUTTON('3'),AT(44,84,16,14),USE(?Key3)
       BUTTON('*'),AT(64,84,16,14),USE(?Mult)
       BUTTON('0'),AT(4,100,36,14),USE(?Key0)
       BUTTON('.'),AT(44,100,16,14),USE(?DecPt)
       BUTTON('/'),AT(64,100,16,14),USE(?Div)
       BUTTON('E&xit'),AT(84,108,24,22),USE(?Exit),STD(STD:Close),KEY(EscKey)
       BUTTON('='),AT(4,116,56,14),USE(?Equal),DEFAULT
       BUTTON('&Neg'),AT(64,116,16,14),USE(?Negate)
     END

result       REAL       ! Current numeric result
mem          REAL       ! Local memory value
new_num      BYTE       ! Does next digit start a number?
op           LONG       ! Last operation executed
val          REAL       ! Value of number displayed, w/o dec or sign
digits       SHORT      ! How many digits in number, w/o dec or sign
sign         SHORT      ! Sign of number displayed
dec          SHORT      ! Dec digits in number displayed
                        !   If -1, no dec hit 
                        !   If 0, dec hit but no digits yet
curfld       SHORT      ! Field num of key hit
numhit       SHORT      ! Number key value
fld          STRING(30)
newfld       STRING(30)
i            SHORT
j            SHORT

  CODE
  fld = ''
  OPEN(Screen)
  curfld = ?Clear
  DO DoClear
  mem = CalcMem
  DO MassageFieldValue
  ALERT(KeyNum0,KeyNum9)
  ALERT(KeyPadNum0,KeyPadNum9)
  ALERT(KeySpace)
  ALERT(KeyEqual)
  ALERT(KeyPlus)
  ALERT(KeyPadPlus)
  ALERT(KeyMinus)
  ALERT(KeyPadMinus)
  ALERT(KeyDiv)
  ALERT(KeyPadDiv)
  ALERT(KeyMult)
  ALERT(KeyPadMult)
  ALERT(KeyDecPt)
  ALERT(KeyPadDecPt)
  ACCEPT
    CASE EVENT()
    OF EVENT:OpenWindow
  COMPILE('SETTHREAD_CALL', USE_THREADS=1)
      SetThreadNumber(CALCULATOR_INDEX)
SETTHREAD_CALL
    OF EVENT:CloseWindow
    OROF EVENT:CloseDown
      BREAK
    OF EVENT:Restore
  COMPILE('RESTORETHREAD_CALL', USE_THREADS=1)
      RestoreThread(CALCULATOR_INDEX)
RESTORETHREAD_CALL
    OF Event:Accepted
      curfld = ?
      DO ProcessKey
    OF Event:PreAlertKey
      CASE KEYCODE()
      OF KeyNum0
      OROF KeyPadNum0
        SelectPress(?Key0)
      OF KeyNum1
      OROF KeyPadNum1
        SelectPress(?Key1)
      OF KeyNum2
      OROF KeyPadNum2
        SelectPress(?Key2)
      OF KeyNum3
      OROF KeyPadNum3
        SelectPress(?Key3)
      OF KeyNum4
      OROF KeyPadNum4
        SelectPress(?Key4)
      OF KeyNum5
      OROF KeyPadNum5
        SelectPress(?Key5)
      OF KeyNum6
      OROF KeyPadNum6
        SelectPress(?Key6)
      OF KeyNum7
      OROF KeyPadNum7
        SelectPress(?Key7)
      OF KeyNum8
      OROF KeyPadNum8
        SelectPress(?Key8)
      OF KeyNum9
      OROF KeyPadNum9
        SelectPress(?Key9)
      OF KeyPlus
      OROF KeyPadPlus
        SelectPress(?Plus)
      OF KeyMinus
      OROF KeyPadMinus
        SelectPress(?Minus)
      OF KeyMult
      OROF KeyPadMult
        SelectPress(?Mult)
      OF KeyDiv
      OROF KeyPadDiv
        SelectPress(?Div)
      OF KeySpace
      OROF KeyEqual
        SelectPress(?Equal)
      OF KeyDecPt
      OROF KeyPadDecPt
        SelectPress(?DecPt)
      END
    END
  END
  DO DoExit

! Massage string value to fit into calculator's window.
MassageFieldValue       ROUTINE
  IF SUB(fld,1,1) = '$' THEN fld = LEFT(SUB(fld, 2, lenclip(fld) - 1)).
  newfld = ''
  j = 0
  LOOP i = 1 TO lenclip(fld)
    CASE fld[i]
    OF '0' TO '9'
    OROF '.'
      j += 1
      newfld[j] = fld[i]
    OF ','  ! We'll take care of commas in formatting
      CYCLE
    ELSE    ! Incoming field is not a valid number
      fld = '' 
      EXIT
    END
  END
  fld = newfld
  val = fld ! If curr field has value
  new_num = TRUE
  DO DecPtOut
  DO ShowVal

! Process the calculator button pressed
ProcessKey              ROUTINE
  numhit = -1
  CASE curfld
  OF ?Key0
    numhit = 0
  OF ?Key1
    numhit = 1
  OF ?Key2
    numhit = 2
  OF ?Key3
    numhit = 3
  OF ?Key4
    numhit = 4
  OF ?Key5
    numhit = 5
  OF ?Key6
    numhit = 6
  OF ?Key7
    numhit = 7
  OF ?Key8
    numhit = 8
  OF ?Key9
    numhit = 9
  OF   ?Plus
  OROF ?Minus
  OROF ?Mult
  OROF ?Div
  OROF ?Equal
  OROF ?Power
    DO DoOp
  OF   ?Sqrt
    DO DoSqrt
  OF   ?DecPt
    DO DoDecPt
  OF   ?ClearEntry
     DO DoClearEntry
  OF   ?Clear
    DO DoClear
  OF EraseField
    DO DoErase
  OF   ?AddMem
  OROF ?SubMem
  OROF ?FlushMem
  OROF ?RecallMem
    DO DoMem
  OF   ?Negate
    DO DoNegate
  OF ?Copy
    SETCLIPBOARD(ValStr)
  OF ?Paste
    fld = CLIP(LEFT(CLIPBOARD()))
    DO MassageFieldValue
  END
  IF numhit >= 0 THEN DO DoNumber.

! Erase last digit
DoErase                 ROUTINE
  val = INT(val/10)
  IF dec >= 0 THEN dec -= 1.
  IF digits > 0 THEN digits -= 1.
  DO ShowVal

! Got first digit for number
NumberBeg               ROUTINE
  IF new_num
    new_num = FALSE
    DO ResetVal
  END

! Append number key
DoNumber                ROUTINE
  DO NumberBeg
  IF dec >= MAX_DEC THEN EXIT.
  IF digits >= MAX_DIGITS THEN EXIT.
  DO FormatValStr
  val = (val * 10) + numhit
  IF dec >= 0 THEN dec += 1.
  digits += 1
  DO ShowVal

! Decimal point key
DoDecPt                 ROUTINE
  DO NumberBeg
  IF dec < 0
    dec = 0
    DO ShowVal
  END

! Process operation: +,-,*,-,=,pow,sqrt
DoOp                    ROUTINE
  IF new_num AND (curfld ~= ?Equal)
  ! Back-to-back non-= ops, with no number in between
  ! User just changed mind
    CASE op ! Previous op
    OF   ?Plus
    OROF ?Minus
    OROF ?Mult
    OROF ?Div
    OROF ?Power
      EXIT
    END
  END
  DO DecPtIn
  CASE op ! Previous op
  ! Perform operation
  OF ?Plus
    result += val
  OF ?Minus
    result -= val
  OF ?Mult
    result *= val
  OF ?Div
    result /= val
  OF ?Power
    result ^= val
  OF ?Equal
    result = val
  END
  new_num = TRUE
  val = result
  DO DecPtOut
  DO ShowVal
  op = curfld

! Put decimal point and sign into value
DecPtIn                 ROUTINE
  IF dec > 0 ! Turn val into floating point number
    val = val/(10^dec)
  END
  IF sign THEN val *= -1.

! Pull out decimal pt and sign from ValStr
! Set: val, dec, sign
DecPtOut                ROUTINE
  IF val < 0
    sign = TRUE
    val *= -1
  ELSE
    sign = FALSE
  END
  ValStr = LEFT(FORMAT(val,@n_30.15)) ! should be big enough for anything
  i = INSTRING('.',ValStr,1) ! find decimal pt
  IF i
    ! Remove trailing zeros
    LOOP j = lenclip(ValStr) TO i BY -1
      IF SUB(ValStr,j,1) ~= '0' THEN BREAK.
    END
    ValStr = SUB(ValStr,1,j)
  END
  dec = lenclip(ValStr) - i  ! Set # of decimal digits
  j = INSTRING('-',ValStr,1) ! Find sign
  digits = lenclip(ValStr)
  IF i THEN digits -= 1.
  IF j THEN digits -= 1.
  IF digits > MAX_DIGITS
    IF dec > (digits - MAX_DIGITS) ! Can trim some dec places
      dec = dec - (digits - MAX_DIGITS)
      digits = MAX_DIGITS
    ELSE
      digits -= dec
      dec = 0
    END
  END
  val *= 10^dec ! Remove decimal point

! Turn val into a string, using sign and decimal point
FormatValStr            ROUTINE
  ValStr = LEFT(FORMAT(val,FMT_MAX_DIGITS))
  digits = lenclip(ValStr) 
  IF digits < dec ! Number is less than .1, prepend 0's
    LOOP i = 1 TO (dec - digits)
      ValStr = '0' & ValStr
    END
    digits = lenclip(ValStr)
  END
  IF dec > 0 ! There is a decimal point
    LOOP i = 1 TO dec ! Make room for decimal point
      ValStr[digits-i+2] = ValStr[digits-i+1]
    END
    ValStr[digits-dec+1] = '.'
  ELSE ! No decimal point, tack it on to end
    ValStr[digits+1] = '.'
  END
  IF SUB(ValStr,1,1) = '.' THEN ValStr = '0' & ValStr. ! Add leading 0
  IF sign THEN ValStr = '-' & LEFT(ValStr). ! Add sign

! Show value on calculator
ShowVal                 ROUTINE
  DO FormatValStr
  ValStr = RIGHT(ValStr)

! Process memory operation
DoMem                   ROUTINE
  CASE curfld
  OF ?AddMem
    DO DecPtIn
    mem += val
    DO DecPtOut
  OF ?SubMem
    DO DecPtIn
    mem -= val
    DO DecPtOut
  OF ?FlushMem
    mem = 0
  OF ?RecallMem
    val = mem
    new_num = TRUE
    DO DecPtOut
    DO ShowVal
  END
  DO ShowMem

! Turn memory indicator on/off
ShowMem                 ROUTINE
  IF roundoff(mem,MAX_DEC_RND) ~= 0
    MemInd = 'M'
  ELSE
    MemInd = ' '
  END

! Process negate sign
DoNegate                ROUTINE
  IF sign
    sign = FALSE
  ELSE
    sign = TRUE
  END
  DO ShowVal

! Reset value to 0
ResetVal                ROUTINE
  val = 0
  dec = -1
  digits = 0
  sign = FALSE

! Process Clear Entry
DoClearEntry            ROUTINE
  DO ResetVal
  DO ShowVal

! Process Clear
DoClear                 ROUTINE
  DO DoClearEntry
  result = 0
  op = ?Equal

! This is a special operation, since it only needs one operand
DoSqrt                  ROUTINE
  DO DecPtIn
  val = SQRT(val)
  DO DecPtOut
  DO ShowVal
  new_num = TRUE

! Exit calculator
DoExit                  ROUTINE
  CalcMem = mem
  COMPILE('CLEARTHREAD_CALL', USE_THREADS=1)
  ClearThreadNumber(CALCULATOR_INDEX)
CLEARTHREAD_CALL
  CLOSE(Screen)
  ALERT ! Turn off alerts
  RETURN 

! This procedure simulates a key press when user presses number on keyboard
SelectPress     PROCEDURE(fld)
  CODE
  SELECT(fld)
  POST(EVENT:Accepted,fld)

  COMPILE('LENCLIP_FUNC', USE_LOCAL_FUNCS=1)
lenclip         FUNCTION(str)
  CODE
  RETURN(LEN(CLIP(str)))
LENCLIP_FUNC

  COMPILE('ROUNDOFF_FUNC', USE_LOCAL_FUNCS=1)
roundoff                FUNCTION(val,rnd)
  CODE
  ! Clarion bug: 
  !   ROUND(.025,.01) returns .02. We want .03.
  !   ROUND(100,100) returns 99. We want 100.
  IF rnd = 0 THEN RETURN(0).
  val /= rnd  
  IF val < 0
    val = INT(val-.5)
  ELSE
    val = INT(val+.5)
  END
  val *= rnd
  RETURN(val)
ROUNDOFF_FUNC

