! ********************************************************************
!               Copyright 1990-1995 by David Lee Cohen
!                    All Rights Reserved
!          Licensed Material - Property of David Lee Cohen
!
!                    Version Date: 5/8/95
! ********************************************************************
!
! ********************************************************************
! Calculator and Calendar 
! ********************************************************************
!
! The Calculator() code implements a calculator featuring:
!   * Unlike many little Windows calculators, you can use the number
!     keys or numeric keypad in addition to the mouse.
!   * Add, subtract, multiply divide operations.
!   * Negate function.
!   * Power and Square Root function.
!   * Copy and Paste between Windows Clipboard.
!   * Single persistant memory.
!
! The Calendar() code implements a calendar featuring:
!   * Full month display.
!   * Flags current date.
!   * Move forward or ahead one month or one year.
!   * Current time.
!
! ********************************************************************
! Cost, Restrictions, and Some Legal Stuff
! ********************************************************************
!
! The Calculator and Calendar code are offered to Clarion for Windows
! developers as shareware. All I ask is a donation of $10 for both.
! Please send payment to:
!   AlleyCat Software, Inc.
!   P.O. Box 14446
!   RTP, NC  27709-4446
!   Compuserve: 70431,132
!   Mastercard/Visa Accepted
!
! What you get is the right to use the code within your applications 
! as you please, with no limits on distribution. You are free to alter
! it in any way you see fit to work with your application.
!
! However, you can not resell or distribute the Calculator and 
! Calendar code as is or with cosmetic changes only, without first 
! receiving permission from David Lee Cohen and AlleyCat Software, Inc.
!
! This code is offered as is, and neither David Lee Cohen or Alleycat
! Software, Inc. is responsible for defects or damages.
! 
! ********************************************************************
! How to Use
! ********************************************************************
!
! The Calculator and Calendar code is simple to use. Each is a self-
! contained chunk of code that doesn't depend on any external 
! variables or procedures. There are no parameters, so they can be
! STARTed in their own threads. 
!
! I'm primarily a hand-coder. Thus, you can use these procedures in
! the Application Generator as Source Procedures, or make some minor
! tweaks and turn them into Code or Procedure Templates.
! 
! There are a few changes you'll need to make to use the Calculator
! and Calendar within your applications:
!
! 1. Change the MEMBER() statement to reference your program.
!    Notice that I've also included a PROGRAM header that can
!    turn these procedures into standalone programs.
!
! 2. Set the IS_PROGRAM conditional compile EQUATE:
!    IS_PROGRAM = 1 means a complete program will be built.
!    IS_PROGRAM = 0 means a MEMBER module is used. You'll need to
!      add the prototype to your program's MAP:
!      MAP
!        MODULE('CALC.CLW')
!          Calculator
!        END
!        MODULE('CALEND.CLW')
!          Calendar
!        END
!      END
!
! 3. Set the USE_LOCAL_FUNCS conditional compile EQUATE:
!    USE_LOCAL_FUNCS = 1 means that a couple local functions are used.
!      I have these defined in libraries, so don't include them here.
!
! 4. Set the USE_THREADS conditional compile EQUATE:
!    USE_THREADS = 1 means that I call my own simple thread management
!      routines. You may want to insert your own thread management 
!      calls at these points instead.
!
! In my applications, I have:
!   IS_PROGRAM = 0
!   USE_LOCAL_FUNCS = 0
!   USE_THREADS = 1
! In your applications, you will likely have:
!   IS_PROGRAM = 0
!   USE_LOCAL_FUNCS = 1
!   USE_THREADS = 1
!   Plus, you will have some minor tweaks to thread code.
! For a standalone program, you would have:
!   IS_PROGRAM = 1
!   USE_LOCAL_FUNCS = 1
!   USE_THREADS = 0
!   (Plus, uncomment out program header below.)
!
! All in all, the Calculator and Calendar procedures should be very
! easy to just plug into your applications. I've been using them
! for some time now - they actually evolved from CDD 3.0 versions.
! Please report any bugs or problems to me, as well as any nifty
! new ideas you dream up.
!
! ********************************************************************
! Files
! ********************************************************************
!
! You should have these files:
!   CALC.CLW - Calculator source code
!   CALC.ICO - Calculator icon
!   CALEND.CLW - Calendar source code
!   CALEND.ICO - Calendar icon
!
! ********************************************************************

! Clarion doesn't allow conditional compilation for PROGRAM/MEMBER.
! So, comment from BEGIN_PROGRAM_HEADER/END_PROGRAM_HEADER for member,
! or comment BEGIN_MEMBER_HEADER/END_MEMBER_HEADER for program.

!BEGIN_PROGRAM_HEADER
!                 TITLE('CALENDAR')
!                 PROGRAM
!
!                 INCLUDE('KEYCODES')
!                 INCLUDE('EQUATES')
!END_PROGRAM_HEADER

!BEGIN_MEMBER_HEADER
                 MEMBER('CAPGNZ')
!END_MEMBER_HEADER

! Is this a standalone program?
! If IS_PROGRAM=1, then Program Header above should be uncommented.
! If IS_PROGRAM=0, then Member Header above should be uncommented.
IS_PROGRAM              EQUATE(0) 

! Use local lenclip(), GetTodaysDate() ?
! I have these in my own libraries, so don't need them.
USE_LOCAL_FUNCS         EQUATE(0) 

! I use a simple thread management scheme here.
! Calls to SetThread(), RestoreThread(), and ClearThreadNumber()
! are used. You can replace these with your own thread mgmt calls, if
! necessary.
USE_THREADS             EQUATE(1) 

  MAP
  COMPILE('LOCAL_FUNCS_DEF', USE_LOCAL_FUNCS=1)
    lenclip(STRING),SHORT
    GetTodaysDate,LONG
LOCAL_FUNCS_DEF
  COMPILE('CALENDAR_DEF', IS_PROGRAM=1)
    Calendar
CALENDAR_DEF
  END

  COMPILE('CURRDATE_DEF', USE_LOCAL_FUNCS=1)
! See the comment at end of module for GetTodaysDate() function.
GLO:CurrDate    LONG
CURRDATE_DEF

  COMPILE('PROGRAM_CODE', IS_PROGRAM=1)
  CODE
  Calendar()
PROGRAM_CODE

Calendar     PROCEDURE

CurrTime     LONG
MonthYear    STRING(15)

  COMPILE('MDI_DEF', USE_THREADS=1)
Screen WINDOW('Calendar'),AT(,,145,161),ICON('CALEND.ICO'),TIMER(100),GRAY,MDI
MDI_DEF
  COMPILE('NONMDI_DEF', USE_THREADS=0)
Screen WINDOW('Calendar'),AT(,,145,161),ICON('CALEND.ICO'),TIMER(100),GRAY
NONMDI_DEF
       STRING(@T6),AT(24,4,,),USE(CurrTime)
       BUTTON('&Today'),AT(104,8,36,14),USE(?Today)
       STRING(@S15),AT(12,16,63,10),USE(MonthYear),CENTER
       STRING('Sun'),AT(4,32,,),CENTER
       STRING('Mon'),AT(24,32,,),CENTER
       STRING('Tue'),AT(44,32,16,10),CENTER
       STRING('Wed'),AT(64,32,,),CENTER
       STRING('Thu'),AT(84,32,16,10),CENTER
       STRING('Fri'),AT(104,32,16,10),CENTER
       STRING('Sat'),AT(124,32,16,10),CENTER
       BUTTON,AT(4,44,16,14),USE(?Day1)
       BUTTON,AT(24,44,16,14),USE(?Day2)
       BUTTON,AT(44,44,16,14),USE(?Day3)
       BUTTON,AT(64,44,16,14),USE(?Day4)
       BUTTON,AT(84,44,16,14),USE(?Day5)
       BUTTON,AT(104,44,16,14),USE(?Day6)
       BUTTON,AT(124,44,16,14),USE(?Day7)
       BUTTON,AT(4,60,16,14),USE(?Day8)
       BUTTON,AT(24,60,16,14),USE(?Day9)
       BUTTON,AT(44,60,16,14),USE(?Day10)
       BUTTON,AT(64,60,16,14),USE(?Day11)
       BUTTON,AT(84,60,16,14),USE(?Day12)
       BUTTON,AT(104,60,16,14),USE(?Day13)
       BUTTON,AT(124,60,16,14),USE(?Day14)
       BUTTON,AT(4,76,16,14),USE(?Day15)
       BUTTON,AT(24,76,16,14),USE(?Day16)
       BUTTON,AT(44,76,16,14),USE(?Day17)
       BUTTON,AT(64,76,16,14),USE(?Day18)
       BUTTON,AT(84,76,16,14),USE(?Day19)
       BUTTON,AT(104,76,16,14),USE(?Day20)
       BUTTON,AT(124,76,16,14),USE(?Day21)
       BUTTON,AT(4,92,16,14),USE(?Day22)
       BUTTON,AT(24,92,16,14),USE(?Day23)
       BUTTON,AT(44,92,16,14),USE(?Day24)
       BUTTON,AT(64,92,16,14),USE(?Day25)
       BUTTON,AT(84,92,16,14),USE(?Day26)
       BUTTON,AT(104,92,16,14),USE(?Day27)
       BUTTON,AT(124,92,16,14),USE(?Day28)
       BUTTON,AT(4,108,16,14),USE(?Day29)
       BUTTON,AT(24,108,16,14),USE(?Day30)
       BUTTON,AT(44,108,16,14),USE(?Day31)
       BUTTON,AT(64,108,16,14),USE(?Day32)
       BUTTON,AT(84,108,16,14),USE(?Day33)
       BUTTON,AT(104,108,16,14),USE(?Day34)
       BUTTON,AT(124,108,16,14),USE(?Day35)
       BUTTON,AT(4,124,16,14),USE(?Day36)
       BUTTON,AT(24,124,16,14),USE(?Day37)
       BUTTON,AT(44,124,16,14),USE(?Day38)
       BUTTON,AT(64,124,16,14),USE(?Day39)
       BUTTON,AT(84,124,16,14),USE(?Day40)
       BUTTON,AT(104,124,16,14),USE(?Day41)
       BUTTON,AT(124,124,16,14),USE(?Day42)
       BUTTON('- Mo'),AT(4,144,20,14),USE(?PrevMonth)
       BUTTON('+ Mo'),AT(28,144,20,14),USE(?NextMonth)
       BUTTON('- Yr'),AT(52,144,20,14),USE(?PrevYear)
       BUTTON('+ Yr'),AT(76,144,20,14),USE(?NextYear)
       BUTTON('E&xit'),AT(104,144,36,14),KEY(EscKey),USE(?Exit),STD(STD:Close)
     END

the_date        LONG
the_month       SHORT
the_day         SHORT
the_year        SHORT

base_col        SHORT  ! Column for first day of month, 0-based
last_day        SHORT  ! Last day of month, in Clarion day format
base_date       LONG   ! Clarion date for first day in month
mo_and_yr       LONG   ! Determines if need to show new month
start_date      LONG
sunday_date     LONG
butnum          SHORT
daystr          STRING(3)
daynum          SHORT

month_grp       GROUP
                  STRING('January   ')
                  STRING('February  ')
                  STRING('March     ')
                  STRING('April     ')
                  STRING('May       ')
                  STRING('June      ')
                  STRING('July      ')
                  STRING('August    ')
                  STRING('September ')
                  STRING('October   ')
                  STRING('November  ')
                  STRING('December  ')
                END
month_array     STRING(10),DIM(12),OVER(month_grp)

  CODE
  COMPILE('CURRDATE_INIT', USE_LOCAL_FUNCS=1)
  IF GLO:CurrDate = 0 THEN GLO:CurrDate = GetTodaysDate().
CURRDATE_INIT
  start_date = GLO:CurrDate
  IF ~INRANGE(start_date,DATE(1,1,1801),DATE(12,31,2099))
    start_date = GetTodaysDate()
  END
  the_date = start_date
  mo_and_yr = 0
  sunday_date = DATE(1,3,1993) ! A known Sunday
  OPEN(Screen)
  DO ShowTime
  DO DoUpdateScreen
  ACCEPT
    CASE EVENT()
    OF EVENT:OpenWindow
  COMPILE('SETTHREAD_CALL', USE_THREADS=1)
      SetThreadNumber(CALENDAR_INDEX)
SETTHREAD_CALL
    OF EVENT:Timer
      Do ShowTime
    OF EVENT:CloseWindow
    OROF EVENT:CloseDown
      BREAK
    OF EVENT:Restore
  COMPILE('RESTORETHREAD_CALL', USE_THREADS=1)
      RestoreThread(CALENDAR_INDEX)
RESTORETHREAD_CALL
    END
    CASE ACCEPTED()
    OF ?Day1 TO ?Day42
      daystr = ?{PROP:Text}
      IF lenclip(daystr) = 0 THEN CYCLE.
      ! Set numeric date:
      !   Month and year are known
      !   Day is current button minus button for first day of month
      !     minus the first day of month plus one
      the_date = DATE(the_month,?-?Day1-base_col+1,the_year)
      DO DoSelect
      DO DoUpdateScreen
    OF ?Today
      DO DoToday
      DO DoUpdateScreen
    OF ?PrevMonth
      Do DoPrevMonth
      DO DoUpdateScreen
    OF ?NextMonth
      Do DoNextMonth
      DO DoUpdateScreen
    OF ?PrevYear
      Do DoPrevYear
      DO DoUpdateScreen
    OF ?NextYear
      Do DoNextYear
      DO DoUpdateScreen
    END
  END
  DO DoExit

! Update screen - this involves checking to see if displayed month/year is
!   same as requested month/year
DoUpdateScreen          ROUTINE
  the_month = MONTH(the_date)
  the_day = DAY(the_date)
  the_year = YEAR(the_date)
  ! Need to show new month?
  IF (the_month + the_year) = mo_and_yr THEN EXIT.
  MonthYear = CLIP(month_array[the_month]) & ' ' & the_year
  base_date = DATE(the_month, 1, the_year)
  base_col = ABS((base_date - sunday_date) % 7)  ! Column for first day
  last_day = DAY(DATE(the_month + 1, 1, the_year) -1) ! Last day of month
  LOOP butnum = ?Day1 TO ?Day42 ! Label buttons
    daynum = butnum - ?Day1 - base_col + 1
    DO DayToStr
    IF DATE(the_month, daystr, the_year) = GetTodaysDate()
      butnum{PROP:fontstyle} = FONT:underline ! Underline today's date
    ELSE
      butnum{PROP:fontstyle} = FONT:regular
    END
    ! If you would like to have 'blank' buttons, so there's a nice
    ! symmetrical array of boxes in the calendar, just comment out
    ! the HIDE and UNHIDE statements below.
    IF lenclip(daystr) > 0
      UNHIDE(butnum)
      butnum{PROP:Text} = daystr
    ELSE
      HIDE(butnum)
    END
  END
  mo_and_yr = the_month + the_year ! Currently showing this month/year

! Show the current time
ShowTime        ROUTINE
  CurrTime = CLOCK()

! Show next month
DoNextMonth     ROUTINE
  the_month += 1
  DO FixupMonth

! Show last month
DoPrevMonth     ROUTINE
  the_month -= 1
  DO FixupMonth

! Show same month, next year
DoNextYear      ROUTINE
  the_year += 1
  DO FixupMonth

! Show same month, last year
DoPrevYear      ROUTINE
  the_year -= 1
  DO FixupMonth

! Pressed a button to select a date
DoSelect        ROUTINE
  IF the_date > 0 
    GLO:CurrDate = the_date
  END

! Show current month/year
DoToday         ROUTINE
  the_date = GetTodaysDate()

! Fix month number up
FixupMonth      ROUTINE
  IF the_month < 1
     the_year -= 1
     the_month = 12
  ELSIF the_month > 12
     the_year += 1
     the_month = 1
  END
  the_date = DATE(the_month, the_day, the_year)

! Exit calendar
DoExit          ROUTINE
  COMPILE('CLEARTHREAD_CALL', USE_THREADS=1)
  ClearThreadNumber(CALENDAR_INDEX)
CLEARTHREAD_CALL
  CLOSE(Screen)
  RETURN

DayToStr        ROUTINE
  IF daynum > last_day THEN daynum = 0. ! Blank button
  IF daynum <= 0 
    daystr = ''
  ELSE
    daystr = FORMAT(daynum,@n2)
  END

  COMPILE('LENCLIP_FUNC', USE_LOCAL_FUNCS=1)
lenclip         FUNCTION(str)
  CODE
  RETURN(LEN(CLIP(str)))
LENCLIP_FUNC

  COMPILE('GETTODAYSDATE_FUNC', USE_LOCAL_FUNCS=1)
GetTodaysDate   FUNCTION
! Might wonder why I wrap this in a function. Well, I have some automated
! tests that I run that depend on the system date. Rather than handle
! changing/restoring the date, I can set a date to use within the program.
  CODE
  RETURN(TODAY())
GETTODAYSDATE_FUNC
