  PROGRAM

!---------------------------------------------------------------------------
! TASKLIST and WIN API Demo Program by:
! Larry Teames
! CREATIVE PC SOLUTIONS
! CIS: 73150,104
! 
! This example program illustrates how to find and "bring to the top" any
! application currently running under Windows.  It builds a List (Queue)
! of all Windows Apps currently executing by using several Windows API
! functions, in conjunction with CW's PROP:Handle facility.  The list
! is then used to "pick" the app to "bring to the top".  Note that for
! ease of use, this app is then placed "on top" of the selected app, thereby
! making the selected app 2nd in the Zorder (the Alt-Tabbing order).
! **************************************************************************
! NOTE THAT A "TYPE INCONSISTENCY" LINK WARNING OCCURS ON "GetWindowTask", and
!      "ShowWindow".  This causes not a problem, and may be ignored.
!---------------------------------------------------------------------------

  INCLUDE('KEYCODES.CLW')
  INCLUDE('EQUATES.CLW')


  MAP
    MAINPROC
    INCLUDE('WINDOWS.CLW')
    MODULE('WIN API PROTOTYPES')
      BringWindowToTop(USHORT),BYTE,PASCAL
      GetWindowText(USHORT,*CSTRING,SHORT),SHORT,RAW,PASCAL
      GetClassName(USHORT,*CSTRING,SHORT),SHORT,RAW,PASCAL
      GetNextWindow(USHORT,BYTE),SHORT,PASCAL
      GetWindowTask(USHORT),USHORT,PASCAL
      FindWindow(*CSTRING,*CSTRING),SHORT,RAW,PASCAL
      PostMessage(USHORT,USHORT,SHORT,ULONG),BYTE,PASCAL
      ShowWindow(USHORT,SHORT),BYTE,PASCAL
    END
  END


WINCLASS    CSTRING(80)    ! Used by FindWindow()
WINTITLE    CSTRING(80)    ! Used by FindWindow()
WINCMD      CSTRING(80)    ! Used by WinExec()

RetHwnd     USHORT
HwndParm    USHORT


! EQUATES used by WinExec for specifying the "state" of the Exec'ed Program's Window
SW_HIDE             EQUATE(0)
SW_SHOWNORMAL       EQUATE(1)
SW_NORMAL           EQUATE(1)
SW_SHOWMINIMIZED    EQUATE(2)
SW_SHOWMAXIMIZED    EQUATE(3)
SW_MAXIMIZE         EQUATE(3)
SW_SHOWNOACTIVATE   EQUATE(4)
SW_SHOW             EQUATE(5)
SW_MINIMIZE         EQUATE(6)
SW_SHOWMINNOACTIVE  EQUATE(7)
SW_SHOWNA           EQUATE(8)
SW_RESTORE          EQUATE(9)

WM_Close            EQUATE(0010h)


  CODE
  MAINPROC                       ! Do the Main Procedure



MAINPROC       PROCEDURE

HwndQUEUE      QUEUE,PRE(HWQ)    ! Queue of Current Tasks
Hwnd             USHORT            ! Window Handle
Htask            USHORT            ! Task Handle
Title            STRING(50)        ! Window Title
Class            STRING(200)       ! Window Class
               END


MainWin WINDOW('TASK SWITCHING DEMO'),AT(1,1,268,157),SYSTEM,RESIZE
       TOOLBAR,AT(0,0,268,16)
         BUTTON('&Close Control Panel'),AT(155,1,,),USE(?CloseCP)
         BUTTON('&ReLoad Task List'),AT(2,1,,),USE(?LoadAllTasks)
         BUTTON('&Find Control Panel'),AT(77,1,,),USE(?FindCP)
       END
       LIST,AT(5,3,259,133),ALRT(EnterKey),USE(?List),HVSCROLL,FORMAT('[26L(2)|~Hwnd~L(2)@p<<<<<<<<#p@](26)M[26L(2)|~Task~L(2)@P<<<<<<<<#P@]M[101L(2)|M~Window Title~L(2)@s60@](124)M[31L(2)~Class Name~L(2)@s200@]M'), |
           FROM(HwndQUEUE)
     END



  CODE
  OPEN(MainWin)

  ?CloseCP{PROP:Hide} = True      ! Hide the "Close CP" button

  ACCEPT
    ALERT()
    CASE EVENT()
      OF EVENT:OpenWindow         ! When window 1st opens:
        DO LoadTaskQueue          ! go load the Queue of Tasks,
        SELECT(?List)             ! then give focus to the List
        PRESSKEY(DownKey)            ! and cause the 1st entry to hilight
        CYCLE

      OF EVENT:Accepted
      OROF EVENT:AlertKey         ! Check for Alerted Key also
        CASE FIELD()
          OF ?FindCP                    ! When asked to Find "Control Panel":
            WINCLASS = 'CtlPanelClass'  ! set the "Class" name,
            WINTITLE = 'Control Panel'  ! set the "Title" name,
            RetHwnd = FindWindow(WINCLASS, WINTITLE) ! Find Control Panel
            IF ~RetHwnd                         ! If not currently running
              WinCmd = 'CONTROL.EXE'            ! set the comand string,
              E# = WinExec(WinCmd, SW_SHOW)     ! then start CP "normal" size
              IF E# < 32                        ! If an error occurred
                R# = MESSAGE('Sorry, Can''t start Control Panel','ERROR',ICON:EXCLAMATION) ! let the user know
                CYCLE                           ! then continue
              END
            END
            RetHwnd = FindWindow(WINCLASS, WINTITLE)   ! Find Control Panel
            x# = BringWindowToTop(RetHwnd)             ! make CP visible,
            x# = ShowWindow(RetHwnd,SW_ShowMinimized)  ! Minimize it
            x# = BringWindowToTop(TARGET{PROP:Handle}) ! then put this task back on top
            ?CloseCP{PROP:Hide} = False                ! Show the "Close" button
            DO LoadTaskQueue                           ! Reload the Queue
            SELECT(?List)                              ! Now shows Control Panel
            CYCLE


          OF ?CloseCP                   ! When asked to Close "Control Panel":
            WINCLASS = 'CtlPanelClass'  ! set the "Class" name,
            WINTITLE = 'Control Panel'  ! set the "Title" name,
            RetHwnd = FindWindow(WINCLASS, WINTITLE) ! Find Control Panel
            IF RetHwnd > 0                           ! If CP is currently running
              x# = ShowWindow(RetHwnd,SW_Restore)    ! Restore it
              x# = PostMessage(RetHwnd,WM_Close,0,0) ! Close it (and any children)
              YIELD                                  ! YIELD to allow immediate close
            END
            x# = BringWindowToTop(TARGET{PROP:Handle}) ! then put this task on top
            DO LoadTaskQueue                           ! Reload the Queue
            ?CloseCP{PROP:Hide} = True                 ! Hide the "Close" Button
            SELECT(?List)                              ! Now shows Control Panel
            CYCLE


          OF ?LoadAllTasks        ! When asked to reload Task Queue:
            DO LoadTaskQueue      ! go load the Queue of Tasks,
            SELECT(?List)         ! then give focus to the List
            CYCLE

          OF ?List
            CASE KEYCODE()
              OF MOUSELEFT2 OROF ENTERKEY                  ! When user selects a Task:
                GET(HwndQUEUE,CHOICE(?List))               ! get the Queue Record,
                x# = BringWindowToTop(HWQ:Hwnd)            ! make that task visible,
                x# = BringWindowToTop(TARGET{PROP:Handle}) ! then put this task back on top
                DO LoadTaskQueue                           ! Reload in Zorder order
                SELECT(?List)
                CYCLE
            END
        END
    END
  END
  RETURN


LoadTaskQueue    ROUTINE
  FREE(HwndQUEUE)                         ! First, free any existing entries
  RetHwnd = TARGET{PROP:Handle}           ! Get this APP's Window Handle
  DO GetTaskInfo                          ! Go Load the Queue
  LOOP                                    ! Loop thru all tasks
    PrvHwnd# = RetHwnd                    ! Save Prev Window Handle
    RetHwnd = GetNextWindow(PrvHwnd#,2)   ! Get Next Window Handle in ZOrder
    IF ~RetHwnd; BREAK.                   ! If No More Handles, we're done
    DO GetTaskInfo                        ! Get other info, load the Queue
  END



GetTaskInfo      ROUTINE
  Htask# = GetWindowTask(RetHwnd)         ! Get the associated Task Handle
  IF ~Htask#; EXIT.                       ! If not a Task, skip it
  len# = GetWindowText(RetHwnd,WINTITLE,SIZE(WINTITLE)) ! Get Window TITLE
  len# = GetClassName(RetHwnd,WINCLASS,SIZE(WINCLASS))  ! Get Window CLASS
  IF ~WINTITLE; EXIT.                                   ! If no title, skip it
  HWQ:Hwnd = RetHwnd                                    ! Load the Queue info
  HWQ:Htask = Htask#                                    ! Load the Queue info
  HWQ:Title = CLIP(WINTITLE)                            ! Load the Queue info
  HWQ:CLASS = CLIP(WINCLASS)                            ! Load the Queue info
  ADD(HwndQUEUE)                                        ! Add the Queue record


