
/* ================================================================== *
 *                                                                    *
 *   VBAPI.C - Call Visual Basic API within CWVBX.DLL                 *
 *                                                                    *
 *   Author: Robert Pupazzoni, Bobcat Systems [CIS:70441,204]         *
 *                                                                    *
 *   Description:                                                     *
 *                                                                    *
 *     CWVBX.DLL contains a subset of the VB API to enable Clarion    *
 *     for Windows to support VBX controls.  The functions in this    *
 *     file allow a CW program to make calls directly to the embedded *
 *     VB API.                                                        *
 *                                                                    *
 *     This file currently implements only those API calls necessary  *
 *     to handle VB HSZ strings.                                      *
 *                                                                    *
 *     Note: You are not meant to understand this code... it will     *
 *           make no sense whatsoever unless you are intimately       *
 *           familiar with the undocumented internals of both         *
 *           CW and the Visual Basic API.                             *
 *                                                                    *
 *           Luckily, understanding is not necessary to use it!       *
 *                                                                    *
 * ================================================================== */


// -- Visual Basic API Jump Offsets --

#define API_RegisterModel       0x0000
#define API_DerefControl        0x0004
#define API_DefControlProc      0x0008
#define API_FireEvent           0x000C
#define API_SetControlProperty  0x0010
#define API_GetControlProperty  0x0014
#define API_CreateHsz           0x0018
#define API_DestroyHsz          0x001C
#define API_DerefHsz            0x0020
#define API_LockHsz             0x0024
#define API_UnlockHsz           0x0028
#define API_GetControlHwnd      0x002C
#define API_RecreateControlHwnd 0x0030
#define API_GetControlModel     0x0034
#define API_SendControlMsg      0x0038
#define API_ReadFormFile        0x003C
#define API_WriteFormFile       0x0040
#define API_SuperControlProc    0x0044
#define API_GetMode             0x0048
#define API_GetHInstance        0x004C
#define API_CreateHlstr         0x0050
#define API_SetHlstr            0x0054
#define API_DestroyHlstr        0x0058
#define API_DerefHlstr          0x005C
#define API_GetHlstrLen         0x0060
#define API_GetHwndControl      0x0064
#define API_XPixelsToTwips      0x0068
#define API_YPixelsToTwips      0x006C
#define API_XTwipsToPixels      0x0070
#define API_YTwipsToPixels      0x0074
#define API_ReadBasicFile       0x0078
#define API_WriteBasicFile      0x007C
#define API_AllocPic            0x0080
#define API_FreePic             0x0084
#define API_GetPic              0x0088
#define API_RefPic              0x008C
#define API_PicFromCF           0x0090
#define API_SetErrorMessage     0x0094
#define API_GetAppTitle         0x0098
#define API_DialogBoxParam      0x009C
#define API_SeekFormFile        0x00A0
#define API_RelSeekFormFile     0x00A4
#define API_GetControlName      0x00A8
#define API_Reserved0           0x00AC
#define API_Reserved1           0x00B0
#define API_Reserved2           0x00B4
#define API_Reserved3           0x00B8
#define API_VBGetVersion        0x00BC

#pragma name(prefix=>"")
#pragma name(upper_case=>on)

// -- Prototype for VB API entry point in CWVBX.DLL

#pragma save
#pragma call(reg_param=>(bx))
#pragma call(reg_return=>(ax,dx,st0))
#pragma call(reg_saved=>(si,di,ds,st1,st2))
extern void far * _AFXVBAPIENTRY( int );
#pragma restore

// -- Define low-level stuff

#pragma save
#pragma call(inline=>on, reg_param=>(ax))
static void push_int( int Arg ) =
{
   0x50    /* push ax */
};
#pragma restore

#pragma save
#pragma call(inline=>on, reg_param=>(ax,es))
static void push_ptr( void *Arg ) =
{
  0x06,   /* push es */
  0x50,   /* push ax */
};
#pragma restore


/* ------------------------------------------------------------------ *
 *   VBDataSeg() - Get VB API Data Segment                            *
 * ------------------------------------------------------------------ */
int VBDataSeg( void )
{
  return( (int) _AFXVBAPIENTRY( API_Reserved0 ) );
}


/* ------------------------------------------------------------------ *
 *   VBCreateHsz() - Call equivalent function in VB API               *
 * ------------------------------------------------------------------ */
long VBCreateHsz( int Segment, long lpszString )
{
  push_int( Segment );
  push_ptr( (void far *) lpszString );
  return( (long) _AFXVBAPIENTRY( API_CreateHsz ) );
}


/* ------------------------------------------------------------------ *
 *   VBLockHsz() - Call equivalent function in VB API                 *
 * ------------------------------------------------------------------ */
long VBLockHsz( long hszString )
{
  push_ptr( (void far *) hszString );
  return( (long) _AFXVBAPIENTRY( API_LockHsz ) );
}


/* ------------------------------------------------------------------ *
 *   VBUnlockHsz() - Call equivalent function in VB API               *
 * ------------------------------------------------------------------ */
void VBUnlockHsz( long hszString )
{
  push_ptr( (void far *) hszString );
  _AFXVBAPIENTRY( API_UnlockHsz );
}


/* ------------------------------------------------------------------ *
 *   VBDerefHsz() - Call equivalent function in VB API                *
 * ------------------------------------------------------------------ */
long VBDerefHsz( long hszString )
{
  push_ptr( (void far *) hszString );
  return( (long) _AFXVBAPIENTRY( API_DerefHsz ) );
}


/* ------------------------------------------------------------------ *
 *   VBDestroyHsz() - Call equivalent function in VB API              *
 * ------------------------------------------------------------------ */
void VBDestroyHsz( long hszString )
{
  push_ptr( (void far *) hszString );
  _AFXVBAPIENTRY( API_DestroyHsz );
}

