
!====================================================================
!
!  VBXMETHD.CLW - Call VBX methods from CW
!
!  Author: Robert Pupazzoni, Bobcat Systems [70441,204]
!
!  Notes:
!
!     This file implements all the standard VBX method calls defined
!     by Visual Basic.  However, please note that:
!
!       (1) A given VBX may or may not support a given method.  Refer
!           to the documentation which came with the VBX.
!
!       (2) Some methods, while supported by the VBX, may not be
!           fully supported by CW.  For example, ZOrder().
!
!       (3) Some VBX methods, are implmented here just for the sake of
!           completeness.  For example, the effect of the Move() method
!           is probably better and easier accomplished the CW
!           SETPOSITION() procedure, or the property {} syntax.
!
!====================================================================

VBXMethd        PROGRAM


!--------------------------------------------------------------------
!  Equates
!--------------------------------------------------------------------

                INCLUDE( 'EQUATES.CLW' )

eVBM_Base       EQUATE(EVENT:User + 0C00H)      ! VBX Messages
eVBM_Method     EQUATE(eVBM_Base  + 0011H)      ! 

eMETH_ADDITEM   EQUATE( 01H )                   ! Method Indices
eMETH_DELITEM   EQUATE( 02H )                   !
eMETH_REFRESH   EQUATE( 03H )                   !
eMETH_MOVE      EQUATE( 04H )                   !
eMETH_DRAG      EQUATE( 05H )                   !
eMETH_LINKSEND  EQUATE( 06H )                   !
eMETH_ZORDER    EQUATE( 07H )                   !
eMETH_CLEAR     EQUATE( 0AH )                   !

!--------------------------------------------------------------------
!  Global Map
!--------------------------------------------------------------------

                MAP
                  VBXAddItem( SHORT, STRING, SHORT )
                  VBXDrag( SHORT, <SHORT> )
                  VBXClear( SHORT )
                  VBXLinkSend( SHORT )
                  VBXMove( SHORT, SHORT, <SHORT>, <SHORT>, <SHORT> )
                  VBXRefresh( SHORT )
                  VBXRemoveItem( SHORT, SHORT )
                  VBXZOrder( SHORT, SHORT )
                  MODULE('')
                    VBDataSeg(), SHORT
                    VBCreateHsz( SHORT, LONG ), LONG
                    VBLockHsz( LONG ), LONG
                    VBUnlockHsz( LONG )
                    VBDerefHsz( LONG ), LONG
                    VBDestroyHsz( LONG )
                  END
                  MODULE('')
                    SendMessage(USHORT, USHORT, USHORT, LONG), LONG, RAW, PASCAL
                  END
                END

!--------------------------------------------------------------------
!  Main Program
!--------------------------------------------------------------------

  CODE
  RETURN


! ========================================================================
!   Call VBX AddItem() method
! ========================================================================
VBXAddItem      PROCEDURE( isField, sItem, isIndex )

                ! Locals:
hCtrl           USHORT                          ! Control handle
csItem          CSTRING( 64 )                   ! Temporary storage

gParms          GROUP,PRE(GAP)                  ! Method parameter structure:
ilArgs            LONG(3)                       !   # of args
hszItem           LONG                          !   Handle to VB HSZ string
ilIndex           LONG                          !   Insertion index
                END

lpParms         LONG                            ! Ptr to parameter structure
isVBSeg         SHORT                           ! VB data segment

  CODE
  csItem  = sItem
  isVBSeg = VBDataSeg()

  GAP:hszItem = VBCreateHsz(isVBSeg,  ADDRESS(csItem))
  GAP:ilIndex = isIndex
  IF GAP:hszItem
    hCtrl   = isField{PROP:Handle}
    lpParms = ADDRESS( gParms )
    Junk#   = SendMessage(hCtrl, eVBM_METHOD, eMETH_ADDITEM, lpParms)
    VBDestroyHsz( GAP:hszItem )
  END
  RETURN


! ========================================================================
!   Call VBX Clear() method
! ========================================================================
VBXClear        PROCEDURE( isField )

                ! Locals:
hCtrl           USHORT                          ! Control handle

  CODE
  hCtrl = isField{PROP:Handle}
  Junk# = SendMessage(hCtrl, eVBM_METHOD, eMETH_CLEAR, 0)
  RETURN


! ========================================================================
!   Call VBX Drag() method
! ========================================================================
VBXDrag         PROCEDURE( isField, isCmd )

                ! Locals:
hCtrl           USHORT                          ! Control handle

gParms          GROUP,PRE(GAP)                  ! Method parameter structure:
ilArgs            LONG(2)                       !   # of args
ilCmd             LONG                          !   Command code
                END

lpParms         LONG                            ! Ptr to parameter structure

  CODE
  IF OMITTED( 2 )
    GAP:ilArgs = 1
   ELSE
    GAP:ilCmd = isCmd
  END
  hCtrl   = isField{PROP:Handle}
  lpParms = ADDRESS( gParms )
  Junk#   = SendMessage(hCtrl, eVBM_METHOD, eMETH_DRAG, lpParms)
  RETURN


! ========================================================================
!   Call VBX LinkSend() method
! ========================================================================
VBXLinkSend     PROCEDURE( isField )

                ! Locals:
hCtrl           USHORT                          ! Control handle

  CODE
  hCtrl = isField{PROP:Handle}
  Junk# = SendMessage(hCtrl, eVBM_METHOD, eMETH_LINKSEND, 0)
  RETURN


! ========================================================================
!   Call VBX Move() method
! ========================================================================
VBXMove        PROCEDURE( isField, isXPos, isYPos, isWidth, isHeight )

                ! Locals:
hCtrl           USHORT                          ! Control handle

gParms          GROUP,PRE(GAP)                  ! Method parameter structure:
ilArgs            LONG(5)                       !   # of args
ilXPos            LONG                          !   Positions
ilYPos            LONG                          !
ilWidth           LONG                          !
ilHeight          LONG                          !
                END

lpParms         LONG                            ! Ptr to parameter structure

  CODE
  IF OMITTED( 5 )
    GAP:ilArgs = 4
   ELSE
    GAP:ilXPos = isHeight
  END
  IF OMITTED( 4 )
    GAP:ilArgs = 3
   ELSE
    GAP:ilXPos = isWidth
  END
  IF OMITTED( 3 )
    GAP:ilArgs = 2
   ELSE
    GAP:ilXPos = isYPos
  END
  GAP:ilXPos = isXPos
  hCtrl      = isField{PROP:Handle}
  lpParms    = ADDRESS( gParms )
  Junk#      = SendMessage(hCtrl, eVBM_METHOD, eMETH_MOVE, lpParms)
  RETURN


! ========================================================================
!   Call VBX Refresh() method
! ========================================================================
VBXRefresh      PROCEDURE( isField )

                ! Locals:
hCtrl           USHORT                          ! Control handle

  CODE
  hCtrl = isField{PROP:Handle}
  Junk# = SendMessage(hCtrl, eVBM_METHOD, eMETH_REFRESH, 0)
  RETURN


! ========================================================================
!   Call VBX RemoveItem() method
! ========================================================================
VBXRemoveItem   PROCEDURE( isField, isIndex )

                ! Locals:
hCtrl           USHORT                          ! Control handle

gParms          GROUP,PRE(GAP)                  ! Method parameter structure:
ilArgs            LONG(2)                       !   # of args
ilIndex           LONG                          !   Insertion index
                END

lpParms         LONG                            ! Ptr to parameter structure

  CODE
  GAP:ilIndex = isIndex
  hCtrl   = isField{PROP:Handle}
  lpParms = ADDRESS( gParms )
  Junk#   = SendMessage(hCtrl, eVBM_METHOD, eMETH_DELITEM, lpParms)
  RETURN


! ========================================================================
!   Call VBX ZOrder() method
! ========================================================================
VBXZOrder       PROCEDURE( isField, isFlag )

                ! Locals:
hCtrl           USHORT                          ! Control handle

gParms          GROUP,PRE(GAP)                  ! Method parameter structure:
ilArgs            LONG(2)                       !   # of args
ilFlag            LONG                          !   Position flag
                END

lpParms         LONG                            ! Ptr to parameter structure

  CODE
  GAP:ilFlag = isFlag
  hCtrl      = isField{PROP:Handle}
  lpParms    = ADDRESS( gParms )
  Junk#      = SendMessage(hCtrl, eVBM_METHOD, eMETH_ZORDER, lpParms)
  RETURN

